import unittest
import time
import os
from nose.tools import assert_raises, assert_almost_equal
from numpy.testing import assert_array_almost_equal
import h5py

from brownian import bayes
from brownian import silentremove, directory



class testACmdStan(unittest.TestCase):
    def setUp(self):
        self.startTime = time.time()
        self.fh = h5py.File(directory+'/ex/brownian-red.h5', 'r')

    def test_integration_cmd_stan(self):
        f = self.fh['x'][:]
        PSD = self.fh['y'][:]
        m = (f > 69500) & (f < 71500)
        d = bayes.np2data(f, PSD, 32, fmin=70400, fmax=70700,
              kc=3.5, Q=20000, sigma_Q=15000, sigma_kc=2.5,
              Pdet=1e-8, sigma_Pdet=3e-8, # Pdet, sigma_Pdet not required
             )
        df_samples = bayes.cmdstan_sample(d, 10, 2,)
        ppb = bayes.PlotCmdStanBrownian(d, df_samples, 'test-cmdstan')

    def tearDown(self):
        self.fh.close()
        silentremove('gamma-1.csv')
        silentremove('gamma-2.csv')
        silentremove('data.dump')
        t = time.time() - self.startTime
        print("%s: %.3f" % (self.id(), t))

class testBayes(unittest.TestCase):
    @unittest.skip("Skipping pymc3: cmdstan seems to work well")
    def setUp(self):
        self.startTime = time.time()
        self.fh = h5py.File(directory+'/ex/brownian-red.h5', 'r')
    def test_integration(self):
        """Integration test: data extraction, sampling, and creation of plotting object."""
        d = bayes.fh2data(self.fh, 70521, 70601, 3.5, 20000,
              sigma_Q=15000, Pdet=1e-8,
              sigma_Pdet=3e-8, sigma_kc=2.5)
        traces = bayes.sample_pymc3(d, samples=10, njobs=1)
        ppb = bayes.PlotPyMCBrownian(d, traces, 'test-data')

    def tearDown(self):
        self.fh.close()
        t = time.time() - self.startTime
        print("%s: %.3f" % (self.id(), t))







"""
This module collects tools for unit testing the functions in the
``brownian`` package. Unit tests were developed using the ``unittest``
package. To run the unit tests, open a terminal in the ``brownian`` directory
and run either::

    python -m unittest discover
    (or python -m unittest discover --verbose)

or::

    python setup.py test

In some of the unit tests, we compare two ``numpy`` arrays.  In developing this
comparison we found the stackoverflow discussion "Comparing numpy float arrays
in unit tests" helpful [`link <http://stackoverflow.com/questions/14920837/comparing-numpy-float-arrays-in-unit-tests>`__].

"""

import os
import unittest


def discover():
    """Automatically collect all tests in this folder."""
    return unittest.TestLoader().discover(os.path.dirname(__file__))


def main():
    """Run all tests.

    :return: a :class:`unittest.TestResult` object
    """
    test_runner = unittest.TextTestRunner()
    return test_runner.run(discover())


import numpy as np
import sys

from six import iteritems
from six.moves import zip as izip
from six.moves import xrange
from itertools import chain, repeat, islice

def intersperse(delimiter, seq):
    # http://stackoverflow.com/a/5655803/344821
    return islice(chain.from_iterable(izip(repeat(delimiter), seq)), 1, None)

def _write_vec(vec, output):
    output.write('c(')
    for thing in intersperse(', ', vec):
        output.write(str(thing))
    output.write(')')

def _write_rep(val, output):
    if np.isscalar(val) and np.isreal(val):
        output.write(str(val))
    elif isinstance(val, xrange) and abs(val[0] - val[1]) == 1:
        output.write('{}:{}'.format(val[0], val[-1]))
    else:
        mat = np.asarray(val)
        if mat.ndim == 1:
            _write_vec(mat, output)
        elif mat.ndim > 1:
            output.write('structure(')
            _write_vec(mat.T.flat, output)
            output.write(', .Dim = ')
            _write_vec(mat.shape, output)
            output.write(')')
        else:
            raise TypeError("Don't know how to handle data {!r}".format(val))

def dump_to_rdata(output=sys.stdout, **things):
    assert hasattr(output, 'write')

    for name, val in iteritems(things):
        output.write(name)
        output.write(' <- ')
        _write_rep(val, output)
        output.write('\n')


# if __name__ == '__main__':
    # N = 1024
    # x = np.arange(N, dtype=np.float64)/4 
    # seed = 2354463
    # np.random.seed(seed)
    # data = {
    # 'N': N,
    # 'x': x,
    # 'seed': seed,
    # 'y': np.random.randn(N)*10 + 3*x - 2,
    # 'mu_alpha': 0,
    # 'sigma_alpha': 5,
    # 'mu_beta': 2,
    # 'sigma_beta': 2,
    # 'mu_sigma': 1
    # }
    # with open('data.dump', 'w') as f:
    #     dump_to_rdata(f, **data)